--[[           ]]
--[[ Libraries ]]
--[[           ]]

local L                 = AceLibrary("AceLocale-2.2"):new("Litmus")
local RL                = AceLibrary("Roster-2.1")
local BC                = AceLibrary("Babble-Class-2.2")


--[[          ]]
--[[ Defaults ]]
--[[          ]]

local defaults = 
{
   Lock = false,
   
   ScanUnits = 5,
   ScanMana = 75,
   ScanFrequency = 1,
   
   SelfFilter		= false,
   ClassFilter = {
      ["DRUID"]		= false,
      ["HUNTER"]	= false, 
      ["MAGE"]		= false, 
      ["PALADIN"]	= false, 
      ["PRIEST"]	= false, 
      ["SHAMAN"]	= false, 
      ["WARLOCK"]	= false, 
   },
   
   GroupFilter = {
      [1] = false,
      [2] = false,
      [3] = false,
      [4] = false,
      [5] = false,
      [6] = false,
      [7] = false,
      [8] = false,
   },
   
   DisplayScale = 1,
   DisplayWhen = L["Always"],
   DisplayClassColor = true,
   
   FrameAlpha	= 1,
   FrameColor	= {r = 0.0,g = 0.0,b = 0.0},
   FrameTitleOn = true,
   
   TextColor	= {r = 0.9,g = 0.9,b = 0.9},
   
   BarAlpha		= 1,
   BarStyle		= L["Default"],
   BarTexture	= L["Default"],
   BarWidth		= 140,
   BarHeight	= 14,
   BarSpacing	= 5,
	
	NamePosition	= L["OutsideBar"],
	ManaPosition	= L["OutsideBar"],
	
	x	= UIParent:GetWidth()/2,
	y	= UIParent:GetHeight()/2,
	
	FrameAlpha	= 0.3,
	
	DisplaySubGroup = true,

}


--[[      ]]
--[[ Menu ]]
--[[      ]]

local menu = {
	type = 'group',
	args = {
		
		lock = {
			type	= 'toggle',
			name	= L["Lock"],
			desc	= L["LockDesc"],
			get		= function() return Litmus.db.profile.Lock end,
			set		= function(v) Litmus.db.profile.Lock = not Litmus.db.profile.Lock end,
			order	= 1,
		},
		
		scan = {
			type	= 'group',
			name	= L["ScanOpt"],
			desc	= L["ScanOptDesc"],
			order	= 2,
			args 	= {
				
				units = {
					type	= 'range',
					name	= L["Units"],
					desc	= L["UnitsDesc"],
					get		= function(v) return Litmus.db.profile.ScanUnits end,
					set		= function(v) Litmus.db.profile.ScanUnits = v; Litmus:UpdateFrame(); Litmus:UpdateBars(); end,
					min		= 1.0,
					max		= 15.0,
					step	= 1.0,
					isPercent	= false,
					order	= 3,
				},
				
				mana = {
					type	= 'range',
					name	= L["Mana"],
					desc	= L["ManaDesc"],
					get		= function(v) return Litmus.db.profile.ScanMana end,
					set		= function(v) Litmus.db.profile.ScanMana = v end,
					min		= 5.0,
					max		= 95.0,
					step	= 5.0,
					isPercent	= false,
					order	= 4,
				},
				
				frequency = {
					type	= 'range',
					name	= L["Frequency"],
					desc	= L["FrequencyDesc"],
					get		= function(v) return Litmus.db.profile.ScanFrequency end,
					set		= function(v) Litmus.db.profile.ScanFrequency = v; Litmus:UpdateFrequency(); end,
					min		= 0.2,
					max		= 2.0,
					step	= 0.2,
					isPercent	= false,
					order	= 5,
				},
				
			}
		},
		
		filter = {
			type	= 'group',
			name	= L["FilterOpt"],
			desc	= L["FilterOptDesc"],
			order	= 7,
			args	= {
				
				class = {
					type	= 'group',
					name	= L["ClassOpt"],
					desc	= L["ClassOptDesc"],
					order	= 8,
					args	= {
						
						druid = {
							type	= 'toggle',
							name	= BC["Druid"],
							desc	= L["FilterDruid"],
							get		= function(u) return Litmus.db.profile.ClassFilter["DRUID"] end,
							set		= function(u) Litmus.db.profile.ClassFilter["DRUID"] = not Litmus.db.profile.ClassFilter["DRUID"] end,
							order	= 9,
						},
						
						hunter = {
							type	= 'toggle',
							name	= BC["Hunter"],
							desc	= L["FilterHunter"],
							get		= function(u) return Litmus.db.profile.ClassFilter["HUNTER"] end,
							set		= function(u) Litmus.db.profile.ClassFilter["HUNTER"] = not Litmus.db.profile.ClassFilter["HUNTER"] end,
--~ 							           function(u) eeMon.db.profile.ClassFilter["HUNTER"] = not eeMon.db.profile.ClassFilter["HUNTER"] end,
							order	= 10,
						},
						
						mage = {
							type	= 'toggle',
							name	= BC["Mage"],
							desc	= L["FilterMage"],
							get		= function(u) return Litmus.db.profile.ClassFilter["MAGE"] end,
							set		= function(u) Litmus.db.profile.ClassFilter["MAGE"] = not Litmus.db.profile.ClassFilter["MAGE"] end,
							order	= 11,
						},
						
						paladin = {
							type	= 'toggle',
							name	= BC["Paladin"],
							desc	= L["FilterPaladin"],
							get		= function(u) return Litmus.db.profile.ClassFilter["PALADIN"] end,
							set		= function(u) Litmus.db.profile.ClassFilter["PALADIN"] = not Litmus.db.profile.ClassFilter["PALADIN"] end,
							order	= 12,
						},
						
						priest = {
							type	= 'toggle',
							name	= BC["Priest"],
							desc	= L["FilterPriest"],
							get		= function(u) return Litmus.db.profile.ClassFilter["PRIEST"] end,
							set		= function(u) Litmus.db.profile.ClassFilter["PRIEST"] = not Litmus.db.profile.ClassFilter["PRIEST"] end,
							order	= 13,
						},
						
						shaman = {
							type	= 'toggle',
							name	= BC["Shaman"],
							desc	= L["FilterShaman"],
							get		= function(u) return Litmus.db.profile.ClassFilter["SHAMAN"] end,
							set		= function(u) Litmus.db.profile.ClassFilter["SHAMAN"] = not Litmus.db.profile.ClassFilter["SHAMAN"] end,
							order	= 14,
						},
						
						warlock = {
							type	= 'toggle',
							name	= BC["Warlock"],
							desc	= L["FilterWarlock"],
							get		= function(u) return Litmus.db.profile.ClassFilter["WARLOCK"] end,
							set		= function(u) Litmus.db.profile.ClassFilter["WARLOCK"] = not Litmus.db.profile.ClassFilter["WARLOCK"] end,
							order	= 15,
						},
						
					}
				},
				
				group = {
					type	= 'group',
					name	= L["GroupOpt"],
					desc	= L["GroupOptDesc"],
					order	= 16,
					args	= {
						
						group1 = {
							type	= 'toggle',
							name	= L["Group1"],
							desc	= L["Group1Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[1] end,
							set		= function(u) Litmus.db.profile.GroupFilter[1] = not Litmus.db.profile.GroupFilter[1]; Litmus:AutoSubgroup() end,
							order	= 17,
						},
						
						group2 = {
							type	= 'toggle',
							name	= L["Group2"],
							desc	= L["Group2Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[2] end,
							set		= function(u) Litmus.db.profile.GroupFilter[2] = not Litmus.db.profile.GroupFilter[2]; Litmus:AutoSubgroup() end,
							order	= 18,
						},
						
						group3 = {
							type	= 'toggle',
							name	= L["Group3"],
							desc	= L["Group3Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[3] end,
							set		= function(u) Litmus.db.profile.GroupFilter[3] = not Litmus.db.profile.GroupFilter[3]; Litmus:AutoSubgroup() end,
							order	= 19,
						},
						
						group4 = {
							type	= 'toggle',
							name	= L["Group4"],
							desc	= L["Group4Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[4] end,
							set		= function(u) Litmus.db.profile.GroupFilter[4] = not Litmus.db.profile.GroupFilter[4]; Litmus:AutoSubgroup() end,
							order	= 20,
						},
						
						group5 = {
							type	= 'toggle',
							name	= L["Group5"],
							desc	= L["Group5Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[5] end,
							set		= function(u) Litmus.db.profile.GroupFilter[5] = not Litmus.db.profile.GroupFilter[5]; Litmus:AutoSubgroup() end,
							order	= 21,
						},
						
						group6 = {
							type	= 'toggle',
							name	= L["Group6"],
							desc	= L["Group6Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[6] end,
							set		= function(u) Litmus.db.profile.GroupFilter[6] = not Litmus.db.profile.GroupFilter[6]; Litmus:AutoSubgroup() end,
							order	= 22,
						},
						
						group7 = {
							type	= 'toggle',
							name	= L["Group7"],
							desc	= L["Group7Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[7] end,
							set		= function(u) Litmus.db.profile.GroupFilter[7] = not Litmus.db.profile.GroupFilter[7]; Litmus:AutoSubgroup() end,
							order	= 23,
						},
						
						group8 = {
							type	= 'toggle',
							name	= L["Group8"],
							desc	= L["Group8Desc"],
							get		= function(u) return Litmus.db.profile.GroupFilter[8] end,
							set		= function(u) Litmus.db.profile.GroupFilter[8] = not Litmus.db.profile.GroupFilter[8]; Litmus:AutoSubgroup() end,
							order	= 24,
						},
						
					}
				},
				
				autogroupfilter = {
					type	= 'toggle',
					name	= L["AutoGroupFilter"],
					desc	= L["AutoGroupFilterDesc"],
					get		= function(u) return Litmus.db.profile.AutoGroupFilter end,
					set		= function(u) Litmus.db.profile.AutoGroupFilter = not Litmus.db.profile.AutoGroupFilter; Litmus:AutoSubgroup() end,
					ordner	= 25,
				},
				
				self = {
					type	= 'toggle',
					name	= L["FilterSelf"],
					desc	= L["FilterSelfDesc"],
					get		= function(u) return Litmus.db.profile.SelfFilter end,
					set		= function(u) Litmus.db.profile.SelfFilter = not Litmus.db.profile.SelfFilter end,
					order	= 44,
				},
				
				pet = {
					type	= 'toggle',
					name	= L["FilterPet"],
					desc	= L["FilterPetDesc"],
					get		= function(u) return Litmus.db.profile.PetFilter end,
					set		= function(u) Litmus.db.profile.PetFilter = not Litmus.db.profile.PetFilter end,
					order	= 45,
				},
				
			}
		},
		
		display = {
			type	= 'group',
			name	= L["DisplayOpt"],
			desc	= L["DisplayOptDesc"],
			order	= 26,
			args	= {
				
				when = {
					type		= 'text',
					name		= L["DisplayWhen"],
					desc		= L["DisplayWhenDesc"],
					get			= function(v) return Litmus.db.profile.DisplayWhen end,
					set			= function(v) Litmus.db.profile.DisplayWhen = v; Litmus:CheckVisibility() end,
					validate	= {L["Always"], L["Party"], L["Raid"], L["Never"]},
					order		= 27,
				},
				
				scale = {
					type		= 'range',
					name		= L["DisplayScale"],
					desc		= L["DisplayScaleDesc"],
					get			= function(v) return Litmus.db.profile.DisplayScale end,
					set			= function(v) Litmus.db.profile.DisplayScale = v; Litmus:Scale() end,
					min			= 0.1,
					max			= 2.0,
					step		= 0.1,
					isPercent	= true,
					order		= 28,
				},
				
				frame = {
					type	= 'group',
					name	= L["FrameOpt"],
					desc	= L["FrameOptDesc"],
					order	= 29,
					args	= {
						
						alpha = {
							type		= 'range',
							name		= L["FrameAlpha"],
							desc		= L["FrameAlphaDesc"],
							get			= function(v) return Litmus.db.profile.FrameAlpha end,
							set			= function(v) Litmus.db.profile.FrameAlpha = v; Litmus:UpdateFrame() end,
							min			= 0.0,
							max			= 1.0,
							step		= 0.05,
							isPercent	= true,
							order		= 30,
						},
						
						color = {
							type		= 'color',
							name		= L["FrameColor"],
							desc		= L["FrameColorDesc"],
							get			= function(v) return Litmus.db.profile.FrameColor.r, Litmus.db.profile.FrameColor.g, Litmus.db.profile.FrameColor.b end,
							set			= function(nr, ng, nb) Litmus.db.profile.FrameColor = {r = nr, g = ng, b = nb}; Litmus:UpdateFrame() end,
							order		= 31,
						},
						
						title = {
							type	= 'toggle',
							name	= L["FrameTitle"],
							desc	= L["FrameTitleDesc"],
							get		= function(v) return Litmus.db.profile.FrameTitleOn end,
							set		= function(v) Litmus.db.profile.FrameTitleOn = not Litmus.db.profile.FrameTitleOn; Litmus:UpdateFrame() end,
							order	= 32,
						},
						
					}
				},
				
				bars = {
					type	= 'group',
					name	= L["BarsOpt"],
					desc	= L["BarsOptDesc"],
					order	= 33,
					args	= {
						
						alpha = {
							type		= 'range',
							name		= L["BarAlpha"],
							desc		= L["BarAlphaDesc"],
							get			= function(v) return Litmus.db.profile.BarAlpha end,
							set			= function(v) Litmus.db.profile.BarAlpha = v end,
							min			= 0.2,
							max			= 1.0,
							step		= 0.01,
							isPercent	= true,
							order		= 34,
						},
						
						style = {
							type		= 'text',
							name		= L["BarStyle"],
							desc		= L["BarStyleDesc"],
							get			= function(v) return Litmus.db.profile.BarStyle end,
							set			= function(v) Litmus.db.profile.BarStyle = v; Litmus:UpdateBars() end,
							validate	= {L["Default"], L["CTRaid"], L["Class"]},
							order		= 35,
						},
						
						texture = {
							type		= 'text',
							name		= L["BarTexture"],
							desc		= L["BarTextureDesc"],
							get			= function(v) return Litmus.db.profile.BarTexture end,
							set			= function(v) Litmus:SetBarTexture(v); Litmus:UpdateBars() end,
							validate	= {
								L["Default"],
								L["Charcoal"],
								L["Bars"],
								L["Bumps"],
								L["Button"],
								L["BantoBar"],
								L["Cloud"],
								L["Cracked"],
								L["Dabs"],
								L["Diagonal"],
								L["Grid"],
								L["Hatched"],
								L["Perl"],
								L["Otravi"],
								L["Rain"],
								L["Skewed"],
								L["Smooth"],
								L["Smudge"],
								L["Water"],
								L["Wisps"],
								L["GlossNeo"],
								L["GlossMac"]
							},
							order		= 36,
						},
						
						width = {
							type		= 'range',
							name		= L["BarWidth"],
							desc		= L["BarWidthDesc"],
							get			= function(v) return Litmus.db.profile.BarWidth end,
							set			= function(v) Litmus.db.profile.BarWidth = v; Litmus:UpdateFrame(); Litmus:UpdateBars() end,
							min			= 50,
							max			= 200,
							step		= 5,
							isPercent	= false,
							order		= 37,
						},
						
						height = {
							type		= 'range',
							name		= L["BarHeight"],
							desc		= L["BarHeightDesc"],
							get			= function(v) return Litmus.db.profile.BarHeight end,
							set			= function(v) Litmus.db.profile.BarHeight = v; Litmus:UpdateFrame(); Litmus:UpdateBars() end,
							min			= 10,
							max			= 30,
							step		= 1,
							isPercent	= false,
							order		= 38,
						},
						
						spacing = {
							type		= 'range',
							name		= L["BarSpacing"],
							desc		= L["BarSpacingDesc"],
							get			= function(v) return Litmus.db.profile.BarSpacing end,
							set			= function(v) Litmus.db.profile.BarSpacing = v; Litmus:UpdateFrame(); Litmus:UpdateBars() end,
							min			= 0,
							max			= 16,
							step		= 1,
							isPercent	= false,
							order		= 39,
						},
						
					}
				},
				
				namePosition = {
					type		= 'text',
					name		= L["NamePosition"],
					desc		= L["NamePositionDesc"],
					get			= function(v) return Litmus.db.profile.NamePosition end,
					set			= function(v) Litmus.db.profile.NamePosition = v; Litmus:UpdateFrame(); Litmus:UpdateBars() end,
					validate	= {L["OutsideBar"], L["InsideBar"]},
					order		= 40,
				},
				
				manaPercPosition = {
					type		= 'text',
					name		= L["ManaPosition"],
					desc		= L["ManaPositionDesc"],
					get			= function(v) return Litmus.db.profile.ManaPosition end,
					set			= function(v) Litmus.db.profile.ManaPosition = v; Litmus:UpdateFrame(); Litmus:UpdateBars() end,
					validate	= {L["OutsideBar"], L["InsideBar"], L["Hidden"]},
					order		= 41,
				},
				
				text = {
					type	= 'group',
					name	= L["TextOpt"],
					desc	= L["TextOptDesc"],
					order	= 46,
					args	= {
						
						color = {
							type		= 'color',
							name		= L["TextColor"],
							desc		= L["TextColorDesc"],
							get			= function(v) return Litmus.db.profile.TextColor.r, Litmus.db.profile.TextColor.g, Litmus.db.profile.TextColor.b end,
							set			= function(nr, ng, nb) Litmus.db.profile.TextColor = {r = nr, g = ng, b = nb}; Litmus:UpdateFrame() end,
							order		= 47,
						},
						
						reset = {
							type		= 'execute',
							name		= L["TextColorReset"],
							desc		= L["TextColorResetDesc"],
							func		= function(v) Litmus.db.profile.TextColor = {r = 0.9, g = 0.9, b = 0.9}; Litmus:UpdateFrame() end,
							order		= 48,
						},
					},
				},
				
				classColorNames = {
					type		= 'toggle',
					name		= L["DisplayClassColor"],
					desc		= L["DisplayClassColorDesc"],
					get			= function(v) return Litmus.db.profile.DisplayClassColor end,
					set			= function(v) Litmus.db.profile.DisplayClassColor = not Litmus.db.profile.DisplayClassColor end,
					order		= 42,
				},
				
				displaySubGroup = {
					type		= 'toggle',
					name		= L["DisplaySubGroup"],
					desc		= L["DisplaySubGroupDesc"],
					get			= function(v) return Litmus.db.profile.DisplaySubGroup end,
					set			= function(v) Litmus.db.profile.DisplaySubGroup = not Litmus.db.profile.DisplaySubGroup; Litmus:UpdateFrame(); Litmus:UpdateBars() end,
					order		= 49,
				},
			}
		},
		
		test = {
			type	= 'execute',
			name	= L["Test"],
			desc	= L["TestDesc"],
			func	= function(v) Litmus:Test() end,
			order	= 43,
		},
		
	}
}


--[[             ]]
--[[ ACE2 Part 2 ]]
--[[             ]]

Litmus = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDB-2.0", "AceConsole-2.0", "AceHook-2.1", "AceDebug-2.0", "FuBarPlugin-2.0")


--[[                   ]]
--[[ ACE2 Menu Options ]]
--[[                   ]]

Litmus.OnMenuRequest = menu
Litmus.name = L["Litmus"]
Litmus.hasIcon = true
Litmus.defaultMinimapPosition = 40
Litmus.cannotDetachTooltip = true
Litmus.independentProfile = true
Litmus.defaultPosition = "LEFT"
Litmus.hideWithoutStandby = true
Litmus.hasNoColor = true


--[[                ]]
--[[ OnInitialize() ]]
--[[                ]]

function Litmus:OnInitialize()
	self:RegisterDB("LitmusDB")
	self:RegisterDefaults('profile', defaults )
	self:RegisterChatCommand( {L["/Litmus"]}, menu )
	if not FuBar then
		self.OnMenuRequest.args.hide.guiName = "Hide minimap icon"
		self.OnMenuRequest.args.hide.desc = "Hide minimap icon"
	end
end

--[[            ]]
--[[ OnEnable() ]]
--[[            ]]

function Litmus:OnEnable()
   -- Register a change in party/raid roster from our library
   self:RegisterEvent("RosterLib_RosterUpdated")
   -- Build our gui
   self:BuildDisplay()
   -- Check frame visibility, show if necessary
   self:CheckVisibility()
end

--[[             ]]
--[[ OnDisable() ]]
--[[             ]]

function Litmus:OnDisable()
   self:HideGUI()
end


--[[                 ]]
--[[ SetBarTexture() ]]
--[[                 ]]

function Litmus:SetBarTexture(locTextureName)
	local texture
	-- Try to get the variable name for this localized text, which we can use to
	-- do a loopup in our texture table
	if L:HasReverseTranslation(locTextureName) then
		texture = L:GetReverseTranslation(locTextureName)
	-- If we couldn't do that (for whatever reason), we'll just set it to Default
	-- which we know exists.  Should never happen, and would mess up localized text
	-- for non-english.
	else
		texture = "Default"
	end
	Litmus.db.profile.BarTexture = texture
	
end
